@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009 Mike Massonnet <mmassonnet@xfce.org>
+ *  Copyright (c) 2009-2011 Mike Massonnet <mmassonnet@xfce.org>
  *
  *  XML parsing based on Xfce4 Panel:
  *  Copyright (c) 2005 Jasper Huijsmans <jasper@xfce.org>
@@ -35,10 +35,9 @@
 #endif
 
 #include <gio/gio.h>
-#include <exo/exo.h>
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "common.h"
 
@@ -60,8 +59,6 @@ struct _ClipmanActionsPrivate
   GtkWidget            *menu;
 };
 
-static void             clipman_actions_class_init          (ClipmanActionsClass *klass);
-static void             clipman_actions_init                (ClipmanActions *actions);
 static void             clipman_actions_finalize            (GObject *object);
 
 /*
@@ -74,14 +71,6 @@ static gint           __clipman_actions_entry_compare       (gpointer a,
 static gint           __clipman_actions_entry_compare_name  (gpointer a,
                                                              gpointer b);
 static void           __clipman_actions_entry_free          (ClipmanActionsEntry *entry);
-#if !GLIB_CHECK_VERSION (2,16,0)
-static void           __foreach_command_create_mi           (gpointer key,
-                                                             gpointer value,
-                                                             gpointer user_data);
-static void           __foreach_command_write_xml           (gpointer key,
-                                                             gpointer value,
-                                                             gpointer user_data);
-#endif
 
 /*
  * Callbacks declarations
@@ -361,10 +350,10 @@ cb_entry_activated (GtkMenuItem *mi,
 
   DBG ("Execute command `%s'", real_command);
 
-  xfce_exec (real_command, FALSE, FALSE, &error);
+  g_spawn_command_line_async (real_command, &error);
   if (error != NULL)
     {
-      xfce_err (_("Unable to execute the command \"%s\"\n\n%s"), real_command, error->message);
+      xfce_dialog_show_error (NULL, error, _("Unable to execute the command \"%s\"\n\n%s"), real_command, error->message);
       g_error_free (error);
     }
   g_free (real_command);
@@ -434,49 +423,6 @@ __clipman_actions_entry_free (ClipmanActionsEntry *entry)
   g_slice_free (ClipmanActionsEntry, entry);
 }
 
-#if !GLIB_CHECK_VERSION (2,16,0)
-static void
-__foreach_command_create_mi (gpointer key,
-                             gpointer value,
-                             gpointer user_data)
-{
-  gchar *command_name = key;
-  gchar *command = value;
-  GtkWidget *menu = user_data;
-  GtkWidget *mi;
-
-  mi = gtk_menu_item_new_with_label (command_name);
-  g_object_set_data (G_OBJECT (mi), "text", g_object_get_data (G_OBJECT (menu), "text"));
-  g_object_set_data (G_OBJECT (mi), "command", command);
-  g_object_set_data (G_OBJECT (mi), "regex", g_object_get_data (G_OBJECT (menu), "regex"));
-  gtk_container_add (GTK_CONTAINER (menu), mi);
-  g_signal_connect (mi, "activate", G_CALLBACK (cb_entry_activated), NULL);
-}
-
-static void
-__foreach_command_write_xml (gpointer key,
-                             gpointer value,
-                             gpointer user_data)
-{
-  gchar *command_name = key;
-  gchar *command = value;
-  GString *output = user_data;
-  gchar *tmp;
-
-  g_string_append (output, "\t\t\t<command>\n");
-
-  tmp = g_markup_escape_text (command_name, -1);
-  g_string_append_printf (output, "\t\t\t\t<name>%s</name>\n", tmp);
-  g_free (tmp);
-
-  tmp = g_markup_escape_text (command, -1);
-  g_string_append_printf (output, "\t\t\t\t<exec>%s</exec>\n", tmp);
-  g_free (tmp);
-
-  g_string_append (output, "\t\t\t</command>\n");
-}
-#endif
-
 /*
  * Public methods
  */
@@ -512,6 +458,7 @@ clipman_actions_add (ClipmanActions *actions,
   ClipmanActionsEntry *entry;
   GSList *l;
   GRegex *_regex;
+  gchar *regex_anchored;
 
   g_return_val_if_fail (G_LIKELY (action_name != NULL), FALSE);
   g_return_val_if_fail (G_LIKELY (command_name != NULL), FALSE);
@@ -523,7 +470,9 @@ clipman_actions_add (ClipmanActions *actions,
   if (l == NULL)
     {
       /* Validate the regex */
-      _regex = g_regex_new (regex, G_REGEX_CASELESS|G_REGEX_ANCHORED, 0, NULL);
+      regex_anchored = g_strdup_printf ("%s$", regex);
+      _regex = g_regex_new (regex_anchored, G_REGEX_CASELESS|G_REGEX_ANCHORED, 0, NULL);
+      g_free (regex_anchored);
       if (_regex == NULL)
         return FALSE;
 
@@ -531,6 +480,7 @@ clipman_actions_add (ClipmanActions *actions,
 
       entry = g_slice_new0 (ClipmanActionsEntry);
       entry->action_name = g_strdup (action_name);
+      entry->pattern = g_strdup (regex);
       entry->regex = _regex;
       entry->group = 0;
       entry->commands = g_hash_table_new_full ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal,
@@ -750,23 +700,20 @@ clipman_actions_match_with_menu (ClipmanActions *actions,
       mi = gtk_separator_menu_item_new ();
       gtk_container_add (GTK_CONTAINER (actions->priv->menu), mi);
 
-#if GLIB_CHECK_VERSION (2,16,0)
-      GHashTableIter iter;
-      gpointer key, value;
-      g_hash_table_iter_init (&iter, entry->commands);
-      while (g_hash_table_iter_next (&iter, &key, &value))
         {
-          mi = gtk_menu_item_new_with_label ((const gchar *)key);
-          g_object_set_data (G_OBJECT (mi), "text", g_object_get_data (G_OBJECT (actions->priv->menu), "text"));
-          g_object_set_data (G_OBJECT (mi), "command", value);
-          g_object_set_data (G_OBJECT (mi), "regex", entry->regex);
-          gtk_container_add (GTK_CONTAINER (actions->priv->menu), mi);
-          g_signal_connect (mi, "activate", G_CALLBACK (cb_entry_activated), NULL);
+          GHashTableIter iter;
+          gpointer key, value;
+          g_hash_table_iter_init (&iter, entry->commands);
+          while (g_hash_table_iter_next (&iter, &key, &value))
+            {
+              mi = gtk_menu_item_new_with_label ((const gchar *)key);
+              g_object_set_data (G_OBJECT (mi), "text", g_object_get_data (G_OBJECT (actions->priv->menu), "text"));
+              g_object_set_data (G_OBJECT (mi), "command", value);
+              g_object_set_data (G_OBJECT (mi), "regex", entry->regex);
+              gtk_container_add (GTK_CONTAINER (actions->priv->menu), mi);
+              g_signal_connect (mi, "activate", G_CALLBACK (cb_entry_activated), NULL);
+            }
         }
-#else
-      g_object_set_data (G_OBJECT (actions->priv->menu), "regex", entry->regex);
-      g_hash_table_foreach (entry->commands, (GHFunc)__foreach_command_create_mi, actions->priv->menu);
-#endif
 
       mi = gtk_separator_menu_item_new ();
       gtk_container_add (GTK_CONTAINER (actions->priv->menu), mi);
@@ -796,12 +743,12 @@ clipman_actions_load (ClipmanActions *actions)
   GMarkupParseContext *context;
   EntryParser *parser;
 
-  load = g_file_load_contents (actions->priv->file, NULL, &data, &size, NULL, NULL);
+  load = g_file_load_contents (actions->priv->file, NULL, &data, (gsize*)&size, NULL, NULL);
 
   if (!load)
     {
       filename = g_strdup (SYSCONFDIR"/xdg/xfce4/panel/xfce4-clipman-actions.xml");
-      load = g_file_get_contents (filename, &data, &size, NULL);
+      load = g_file_get_contents (filename, &data, (gsize*)&size, NULL);
       g_free (filename);
     }
 
@@ -854,7 +801,7 @@ clipman_actions_save (ClipmanActions *actions)
       g_string_append_printf (output, "\t\t<name>%s</name>\n", tmp);
       g_free (tmp);
 
-      tmp = g_markup_escape_text (g_regex_get_pattern (entry->regex), -1);
+      tmp = g_markup_escape_text (entry->pattern, -1);
       g_string_append_printf (output, "\t\t<regex>%s</regex>\n", tmp);
       g_free (tmp);
 
@@ -862,27 +809,25 @@ clipman_actions_save (ClipmanActions *actions)
 
       g_string_append (output, "\t\t<commands>\n");
 
-#if GLIB_CHECK_VERSION (2,16,0)
-      GHashTableIter iter;
-      gpointer key, value;
-      g_hash_table_iter_init (&iter, entry->commands);
-      while (g_hash_table_iter_next (&iter, &key, &value))
         {
-          g_string_append (output, "\t\t\t<command>\n");
+          GHashTableIter iter;
+          gpointer key, value;
+          g_hash_table_iter_init (&iter, entry->commands);
+          while (g_hash_table_iter_next (&iter, &key, &value))
+            {
+              g_string_append (output, "\t\t\t<command>\n");
 
-          tmp = g_markup_escape_text (key, -1);
-          g_string_append_printf (output, "\t\t\t\t<name>%s</name>\n", tmp);
-          g_free (tmp);
+              tmp = g_markup_escape_text (key, -1);
+              g_string_append_printf (output, "\t\t\t\t<name>%s</name>\n", tmp);
+              g_free (tmp);
 
-          tmp = g_markup_escape_text (value, -1);
-          g_string_append_printf (output, "\t\t\t\t<exec>%s</exec>\n", tmp);
-          g_free (tmp);
+              tmp = g_markup_escape_text (value, -1);
+              g_string_append_printf (output, "\t\t\t\t<exec>%s</exec>\n", tmp);
+              g_free (tmp);
 
-          g_string_append (output, "\t\t\t</command>\n");
+              g_string_append (output, "\t\t\t</command>\n");
+            }
         }
-#else
-      g_hash_table_foreach (entry->commands, (GHFunc)__foreach_command_write_xml, output);
-#endif
 
       g_string_append (output, "\t\t</commands>\n");
 
@@ -906,7 +851,7 @@ clipman_actions_save (ClipmanActions *actions)
  *
  */
 ClipmanActions *
-clipman_actions_get ()
+clipman_actions_get (void)
 {
   static ClipmanActions *singleton = NULL;
 
