@@ -1,7 +1,7 @@
 /*
  *  common.h -- defines default values to use between the components
  *
- *  Copyright (c) 2009 Mike Massonnet <mmassonnet@xfce.org>
+ *  Copyright (c) 2009-2011 Mike Massonnet <mmassonnet@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,12 +26,20 @@
 #define DEFAULT_MAX_TEXTS_IN_HISTORY                    10
 #define DEFAULT_MAX_IMAGES_IN_HISTORY                   1
 #define DEFAULT_SAVE_ON_QUIT                            TRUE
+#define DEFAULT_REVERSE_ORDER                           FALSE
 
 /* Collector */
 #define DEFAULT_ADD_PRIMARY_CLIPBOARD                   FALSE
 #define DEFAULT_HISTORY_IGNORE_PRIMARY_CLIPBOARD        TRUE
 #define DEFAULT_ENABLE_ACTIONS                          FALSE
 
+/*
+ * Modes for paste-on-activate
+ */
+#define PASTE_INACTIVE  0
+#define PASTE_CTRL_V    1
+#define PASTE_SHIFT_INS 2
+
 /*
  * Selection for the popup command
  */
